using System;
using System.Globalization;
using System.Reflection;
using System.Resources;
using System.Runtime.InteropServices;
using System.Threading;

/// <summary>
///  Poznate anti-debug metode.
/// </summary>
public sealed class AntiDebug
{
    private AntiDebug() { }

    #region "dll importi"
    [System.Runtime.InteropServices.DllImport("kernel32.dll")]
    static extern IntPtr LoadLibrary(string lpFileName);

    [System.Runtime.InteropServices.DllImport("kernel32.dll")]
    static extern bool FreeLibrary(IntPtr hModule);

    [System.Runtime.InteropServices.DllImport("kernel32.dll", CharSet = CharSet.Ansi, ExactSpelling = true)]
    static extern IntPtr GetProcAddress(IntPtr hModule, string procName);

    [System.Runtime.InteropServices.DllImport("kernel32.dll", SetLastError = true)]
    static extern bool CloseHandle(int hFile);

    [System.Runtime.InteropServices.DllImport("kernel32", SetLastError = true)]
    static extern int CreateFile(string filename,
                                    uint desiredAccess,
                                    uint shareMode,
                                    IntPtr attributes,
                                    uint creationDisposition,
                                    uint flagsAndAttributes,
                                    IntPtr templateFile);

    #endregion

    #region Win32 konstante
    private const uint GENERIC_WRITE = 0x40000000;
    private const uint GENERIC_READ = 0x80000000;

    private const uint FILE_SHARE_READ = 0x00000001;
    private const uint FILE_SHARE_WRITE = 0x00000002;

    private const uint OPEN_EXISTING = 3;
    private const uint FILE_ATTRIBUTE_NORMAL = 0x00000080;

    private const int INVALID_HANDLE_VALUE = -1;

    #endregion

    public static bool ChechBadDriverCreateFile()
    {
        string[] bad_drivers = { @"\\.\FILEMON", @"\\.\REGMON", @"\\.\ICEDUMP" };

        foreach ( string device in bad_drivers ) {
            int hmod = CreateFile(device,                   // File name
                GENERIC_READ | GENERIC_WRITE,               // Desired access
                FILE_SHARE_READ | FILE_SHARE_WRITE,         // Share mode
                ( IntPtr )0,                                // Attributes
                OPEN_EXISTING,                              // Creation disposition
                FILE_ATTRIBUTE_NORMAL,                      // Flags and attributes
                ( IntPtr )0);                               // Template file

            if ( hmod != INVALID_HANDLE_VALUE ) {
                CloseHandle(hmod);
                return true;
            }
        }
        return false;
    }

    /// <summary>
    /// Provjerava da li je SoftIce/TRW2000 driver loadan preko CreateFileA() trika.
    /// </summary>
    public static bool CheckSysDebuggerCreateFile()
    {
        string[] bad_drivers = { @"\\.\SICE", @"\.\SICE", @"\\.\NTICE", @"\.\NTICE", @"\\.\TRW", @"\\.\TRWDEBUG" };

        foreach ( string device in bad_drivers ) {
            int hmod = CreateFile(device,             // File name
                GENERIC_READ | GENERIC_WRITE,         // Desired access
                FILE_SHARE_READ | FILE_SHARE_WRITE,   // Share mode
                ( IntPtr )0,                          // Attributes
                OPEN_EXISTING,                        // Creation disposition
                FILE_ATTRIBUTE_NORMAL,                // Flags and attributes
                ( IntPtr )0);                         // Template file

            if ( hmod != INVALID_HANDLE_VALUE ) {
                CloseHandle(hmod);
                return true;
            }
        }

        return false;
    }

    /// <summary>
    /// Provjerava da li je postavljen bpx na danom Win32 API-ju.
    /// </summary>
    public static bool ProvjeriBpx(string ImeDLL, string API)
    {
        bool ima_bpx = false;
        byte[] temp_buf = new byte[13];

        IntPtr hLib = LoadLibrary(ImeDLL);
        IntPtr adresa_API = GetProcAddress(hLib, API);

        if ( hLib.ToInt32() == 0 || adresa_API.ToInt32() == 0 ) {
            ResourceManager stringManager = new ResourceManager("Project7.Properties.Resources", Assembly.GetExecutingAssembly());
            System.Diagnostics.Debugger.Log(0, "1",
                                            String.Format(CultureInfo.InvariantCulture, stringManager.GetString("badAPICall", Thread.CurrentThread.CurrentCulture),
                                            API, ImeDLL));
            return false;
        }

        Marshal.Copy(adresa_API, temp_buf, 0, temp_buf.Length);

        for ( int i = 0; i < temp_buf.Length - 1; i++ )
            if ( temp_buf[i] == 0xCC ) ima_bpx = true;

        FreeLibrary(hLib);
        return ima_bpx;
    }
}
